@@ -1,4 +1,4 @@
-# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2013-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -21,74 +21,82 @@ from cubes.processing.views.forms import RunEditForm
 
 class RunFormsTC(CubicWebTC):
 
-    xpath_hidden_pdef = ('input[ starts-with(@name, "param_def-subject:") and '
-                         '@type="hidden" ]')
+    xpath_hidden_pdef = (
+        'input[ starts-with(@name, "param_def-subject:") and ' '@type="hidden" ]'
+    )
 
     def setup_database(self):
         with self.admin_access.repo_cnx() as cnx:
-            self.exe_eid = cnx.create_entity("Executable", name=u"exe").eid
+            self.exe_eid = cnx.create_entity("Executable", name="exe").eid
             cnx.commit()
 
     def build_view(self):
         """Build the Run creation view"""
         with self.admin_access.web_request(
-                __linkto=u'executable:%s:subject' % self.exe_eid) as req:
-            return self.view('creation', req=req, template=None, etype='Run')
+            __linkto="executable:%s:subject" % self.exe_eid
+        ) as req:
+            return self.view("creation", req=req, template=None, etype="Run")
 
     def test_build_view(self):
         """Basic check from the build_view method"""
         html = self.build_view()
-        els = html.etree.xpath(
-            '//select[starts-with(@name, "executable-subject")]')
+        els = html.etree.xpath('//select[starts-with(@name, "executable-subject")]')
         # No executable selector
         self.assertEqual(len(els), 0)
         # There must be NO '+ add a ParameterValue' link
-        els = html.etree.xpath('//a[ starts-with(@href, '
-                               '"javascript: addInlineCreationForm(" ) ]')
+        els = html.etree.xpath(
+            "//a[ starts-with(@href, " '"javascript: addInlineCreationForm(" ) ]'
+        )
         self.assertEqual(len(els), 0)
         # There must be no "normal" ParamValue field
-        els = html.etree.xpath('//select[ starts-with(@name, '
-                               '"param_def-subject:")]')
+        els = html.etree.xpath("//select[ starts-with(@name, " '"param_def-subject:")]')
         self.assertEqual(len(els), 0)
 
     def test_creation_linkto_noparam(self):
         html = self.build_view()
         # There must be no parameter value field
-        els = html.etree.xpath('//' + self.xpath_hidden_pdef)
+        els = html.etree.xpath("//" + self.xpath_hidden_pdef)
         self.assertEqual(len(els), 0)
 
     def test_creation_linkto_params(self):
         with self.admin_access.repo_cnx() as cnx:
-            for vtype in (u'Float', u'Int', u'String', u'File'):
+            for vtype in ("Float", "Int", "String", "File"):
                 # add a parameter of each type
-                cnx.create_entity("ParameterDefinition",
-                                  name=u"p" + vtype,
-                                  param_type=u"input",
-                                  value_type=vtype,
-                                  parameter_of=self.exe_eid)
+                cnx.create_entity(
+                    "ParameterDefinition",
+                    name="p" + vtype,
+                    param_type="input",
+                    value_type=vtype,
+                    parameter_of=self.exe_eid,
+                )
             cnx.commit()
         html = self.build_view()
         # There must be one field per parameter definition
-        els = html.etree.xpath('//' + self.xpath_hidden_pdef)
+        els = html.etree.xpath("//" + self.xpath_hidden_pdef)
         self.assertEqual(len(els), 4)
 
     def test_unregister(self):
         with self.admin_access.repo_cnx() as cnx:
-            cnx.create_entity("ParameterDefinition", name=u"p",
-                              param_type=u"input", value_type=u"File",
-                              parameter_of=self.exe_eid)
+            cnx.create_entity(
+                "ParameterDefinition",
+                name="p",
+                param_type="input",
+                value_type="File",
+                parameter_of=self.exe_eid,
+            )
             cnx.commit()
         try:
-            self.vreg['forms'].unregister(RunEditForm)
+            self.vreg["forms"].unregister(RunEditForm)
             html = self.build_view()
             # There must be no parameter value field
-            els = html.etree.xpath('//' + self.xpath_hidden_pdef)
+            els = html.etree.xpath("//" + self.xpath_hidden_pdef)
             self.assertEqual(len(els), 0)
         finally:
             # Restore vreg in any case
-            self.vreg['forms'].register(RunEditForm)
+            self.vreg["forms"].register(RunEditForm)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import unittest
+
     unittest.main()
